@@ -4,7 +4,7 @@
 '''
 
 # Import Python Libs
-from __future__ import absolute_import
+from __future__ import absolute_import, unicode_literals, print_function
 import types
 from datetime import datetime
 
@@ -14,15 +14,14 @@ from tests.support.unit import TestCase, skipIf
 from tests.support.mock import (
     MagicMock,
     patch,
-    NO_MOCK,
-    NO_MOCK_REASON
 )
 
 # Import Salt Libs
 import salt.modules.win_system as win_system
+import salt.utils.platform
 
 
-@skipIf(NO_MOCK, NO_MOCK_REASON)
+@skipIf(not salt.utils.platform.is_windows(), 'System is not Windows')
 class WinSystemTestCase(TestCase, LoaderModuleMockMixin):
     '''
         Test cases for salt.modules.win_system
@@ -30,12 +29,19 @@ class WinSystemTestCase(TestCase, LoaderModuleMockMixin):
     def setup_loader_modules(self):
         modules_globals = {}
         if win_system.HAS_WIN32NET_MODS is False:
-            win32api = types.ModuleType('win32api')
+            win32api = types.ModuleType(
+                str('win32api')  # future lint: disable=blacklisted-function
+            )
             now = datetime.now()
             win32api.GetLocalTime = MagicMock(return_value=[now.year, now.month, now.weekday(),
                                                             now.day, now.hour, now.minute,
                                                             now.second, now.microsecond])
             modules_globals['win32api'] = win32api
+            win32net = types.ModuleType(str('win32net'))  # future lint: disable=blacklisted-function
+            win32net.NetServerGetInfo = MagicMock()
+            win32net.NetServerSetInfo = MagicMock()
+            modules_globals['win32net'] = win32net
+
         return {win_system: modules_globals}
 
     def test_halt(self):
@@ -53,7 +59,7 @@ class WinSystemTestCase(TestCase, LoaderModuleMockMixin):
         self.assertEqual(win_system.init(3),
                          'Not implemented on Windows at this time.')
 
-    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
+    @skipIf(not win_system.HAS_WIN32NET_MODS, 'Missing win32 libraries')
     def test_poweroff(self):
         '''
             Test to poweroff a running system
@@ -62,7 +68,7 @@ class WinSystemTestCase(TestCase, LoaderModuleMockMixin):
         with patch.object(win_system, 'shutdown', mock):
             self.assertEqual(win_system.poweroff(), 'salt')
 
-    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
+    @skipIf(not win_system.HAS_WIN32NET_MODS, 'Missing win32 libraries')
     def test_reboot(self):
         '''
             Test to reboot the system
@@ -71,7 +77,7 @@ class WinSystemTestCase(TestCase, LoaderModuleMockMixin):
                    MagicMock(return_value=True)) as shutdown:
             self.assertEqual(win_system.reboot(), True)
 
-    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
+    @skipIf(not win_system.HAS_WIN32NET_MODS, 'Missing win32 libraries')
     def test_reboot_with_timeout_in_minutes(self):
         '''
             Test to reboot the system with a timeout
@@ -82,7 +88,7 @@ class WinSystemTestCase(TestCase, LoaderModuleMockMixin):
             shutdown.assert_called_with(timeout=5, in_seconds=False, reboot=True,
                                         only_on_pending_reboot=False)
 
-    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
+    @skipIf(not win_system.HAS_WIN32NET_MODS, 'Missing win32 libraries')
     def test_reboot_with_timeout_in_seconds(self):
         '''
             Test to reboot the system with a timeout
@@ -93,7 +99,7 @@ class WinSystemTestCase(TestCase, LoaderModuleMockMixin):
             shutdown.assert_called_with(timeout=5, in_seconds=True, reboot=True,
                                         only_on_pending_reboot=False)
 
-    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
+    @skipIf(not win_system.HAS_WIN32NET_MODS, 'Missing win32 libraries')
     def test_reboot_with_wait(self):
         '''
             Test to reboot the system with a timeout and
@@ -106,7 +112,7 @@ class WinSystemTestCase(TestCase, LoaderModuleMockMixin):
             self.assertEqual(win_system.reboot(wait_for_reboot=True), True)
             time.assert_called_with(330)
 
-    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
+    @skipIf(not win_system.HAS_WIN32NET_MODS, 'Missing win32 libraries')
     def test_shutdown(self):
         '''
             Test to shutdown a running system
@@ -115,7 +121,7 @@ class WinSystemTestCase(TestCase, LoaderModuleMockMixin):
                    MagicMock()):
             self.assertEqual(win_system.shutdown(), True)
 
-    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
+    @skipIf(not win_system.HAS_WIN32NET_MODS, 'Missing win32 libraries')
     def test_shutdown_hard(self):
         '''
             Test to shutdown a running system with no timeout or warning
@@ -125,7 +131,7 @@ class WinSystemTestCase(TestCase, LoaderModuleMockMixin):
             self.assertEqual(win_system.shutdown_hard(), True)
             shutdown.assert_called_with(timeout=0)
 
-    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
+    @skipIf(not win_system.HAS_WIN32NET_MODS, 'Missing win32 libraries')
     def test_set_computer_name(self):
         '''
             Test to set the Windows computer name
@@ -144,7 +150,7 @@ class WinSystemTestCase(TestCase, LoaderModuleMockMixin):
                    MagicMock(return_value=False)):
             self.assertFalse(win_system.set_computer_name("salt"))
 
-    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
+    @skipIf(not win_system.HAS_WIN32NET_MODS, 'Missing win32 libraries')
     def test_get_pending_computer_name(self):
         '''
             Test to get a pending computer name.
@@ -160,7 +166,7 @@ class WinSystemTestCase(TestCase, LoaderModuleMockMixin):
                 self.assertEqual(win_system.get_pending_computer_name(),
                                  'salt_pending')
 
-    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
+    @skipIf(not win_system.HAS_WIN32NET_MODS, 'Missing win32 libraries')
     def test_get_computer_name(self):
         '''
             Test to get the Windows computer name
@@ -170,21 +176,22 @@ class WinSystemTestCase(TestCase, LoaderModuleMockMixin):
             self.assertEqual(win_system.get_computer_name(), 'computer name')
             self.assertFalse(win_system.get_computer_name())
 
-    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
+    @skipIf(not win_system.HAS_WIN32NET_MODS, 'Missing win32 libraries')
     def test_set_computer_desc(self):
         '''
             Test to set the Windows computer description
         '''
-        mock = MagicMock(return_value=True)
-        with patch.dict(win_system.__salt__, {'cmd.run': mock}):
-            mock = MagicMock(return_value="Salt's comp")
-            with patch.object(win_system, 'get_computer_desc', mock):
-                self.assertDictEqual(win_system.set_computer_desc(
-                                                                  "Salt's comp"
-                                                                  ),
-                                     {'Computer Description': "Salt's comp"})
+        mock = MagicMock()
+        mock_get_info = MagicMock(return_value={'comment': ''})
+        mock_get_desc = MagicMock(return_value="Salt's comp")
+        with patch('salt.modules.win_system.win32net.NetServerGetInfo', mock_get_info), \
+                patch('salt.modules.win_system.win32net.NetServerSetInfo', mock), \
+                patch.object(win_system, 'get_computer_desc', mock_get_desc):
+            self.assertDictEqual(
+                win_system.set_computer_desc("Salt's comp"),
+                {'Computer Description': "Salt's comp"})
 
-    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
+    @skipIf(not win_system.HAS_WIN32NET_MODS, 'Missing win32 libraries')
     def test_get_computer_desc(self):
         '''
             Test to get the Windows computer description
@@ -195,7 +202,7 @@ class WinSystemTestCase(TestCase, LoaderModuleMockMixin):
             self.assertEqual(win_system.get_computer_desc(), 'salt description')
             self.assertFalse(win_system.get_computer_desc())
 
-    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs w32net and other windows libraries')
+    @skipIf(not win_system.HAS_WIN32NET_MODS, 'Missing win32 libraries')
     def test_join_domain(self):
         '''
             Test to join a computer to an Active Directory domain
@@ -228,7 +235,7 @@ class WinSystemTestCase(TestCase, LoaderModuleMockMixin):
             import re
             self.assertTrue(re.search(r'^\d{2}:\d{2} \w{2}$', win_tm))
 
-    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
+    @skipIf(not win_system.HAS_WIN32NET_MODS, 'Missing win32 libraries')
     def test_set_system_time(self):
         '''
             Test to set system time
@@ -245,7 +252,7 @@ class WinSystemTestCase(TestCase, LoaderModuleMockMixin):
         date = datetime.strftime(datetime.now(), "%m/%d/%Y")
         self.assertEqual(win_system.get_system_date(), date)
 
-    @skipIf(not win_system.HAS_WIN32NET_MODS, 'this test needs the w32net library')
+    @skipIf(not win_system.HAS_WIN32NET_MODS, 'Missing win32 libraries')
     def test_set_system_date(self):
         '''
             Test to set system date
@@ -292,3 +299,42 @@ class WinSystemTestCase(TestCase, LoaderModuleMockMixin):
             ret = win_system.get_hostname()
             self.assertEqual(ret, "MINION")
         cmd_run_mock.assert_called_once_with(cmd="hostname")
+
+    @skipIf(not win_system.HAS_WIN32NET_MODS, 'Missing win32 libraries')
+    def test_get_system_info(self):
+        fields = ['bios_caption', 'bios_description', 'bios_details',
+                  'bios_manufacturer', 'bios_version', 'bootup_state',
+                  'caption', 'chassis_bootup_state', 'chassis_sku_number',
+                  'description', 'dns_hostname', 'domain', 'domain_role',
+                  'hardware_manufacturer', 'hardware_model', 'hardware_serial',
+                  'install_date', 'last_boot', 'name',
+                  'network_server_mode_enabled', 'organization',
+                  'os_architecture', 'os_manufacturer', 'os_name', 'os_type',
+                  'os_version', 'part_of_domain', 'pc_system_type',
+                  'power_state', 'primary', 'processor_cores',
+                  'processor_cores_enabled', 'processor_manufacturer',
+                  'processor_max_clock_speed', 'processors',
+                  'processors_logical', 'registered_user', 'status',
+                  'system_directory', 'system_drive', 'system_type',
+                  'thermal_state', 'total_physical_memory',
+                  'total_physical_memory_raw', 'users', 'windows_directory',
+                  'workgroup']
+        ret = win_system.get_system_info()
+        # Make sure all the fields are in the return
+        for field in fields:
+            self.assertIn(field, ret)
+        # os_type
+        os_types = ['Work Station', 'Domain Controller', 'Server']
+        self.assertIn(ret['os_type'], os_types)
+        domain_roles = ['Standalone Workstation', 'Member Workstation',
+                        'Standalone Server', 'Member Server',
+                        'Backup Domain Controller', 'Primary Domain Controller']
+        self.assertIn(ret['domain_role'], domain_roles)
+        system_types = ['Unspecified', 'Desktop', 'Mobile', 'Workstation',
+                        'Enterprise Server', 'SOHO Server', 'Appliance PC',
+                        'Performance Server', 'Slate', 'Maximum']
+        self.assertIn(ret['pc_system_type'], system_types)
+        warning_states = ['Other', 'Unknown', 'Safe', 'Warning', 'Critical',
+                          'Non-recoverable']
+        self.assertIn(ret['chassis_bootup_state'], warning_states)
+        self.assertIn(ret['thermal_state'], warning_states)
