@@ -9,28 +9,31 @@ import copy
 import os
 
 # Import Salt Testing libs
-from tests.support.unit import TestCase, skipIf
-from tests.support.mock import NO_MOCK, NO_MOCK_REASON, patch, MagicMock
+from tests.support.unit import TestCase
+from tests.support.mock import patch, MagicMock
 from tests.support.mixins import AdaptedConfigurationTestCaseMixin
 from tests.support.helpers import skip_if_not_root
 # Import salt libs
 import salt.minion
 import salt.utils.event as event
-from salt.exceptions import SaltSystemExit
+from salt.exceptions import SaltSystemExit, SaltMasterUnresolvableError
 import salt.syspaths
-import tornado
-import tornado.testing
+import salt.ext.tornado
+import salt.ext.tornado.testing
 from salt.ext.six.moves import range
+import salt.utils.crypt
+import salt.utils.process
 
 
-__opts__ = {}
-
-
-@skipIf(NO_MOCK, NO_MOCK_REASON)
 class MinionTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
+
+    def setUp(self):
+        self.opts = {}
+        self.addCleanup(delattr, self, 'opts')
+
     def test_invalid_master_address(self):
-        with patch.dict(__opts__, {'ipv6': False, 'master': float('127.0'), 'master_port': '4555', 'retry_dns': False}):
-            self.assertRaises(SaltSystemExit, salt.minion.resolve_dns, __opts__)
+        with patch.dict(self.opts, {'ipv6': False, 'master': float('127.0'), 'master_port': '4555', 'retry_dns': False}):
+            self.assertRaises(SaltSystemExit, salt.minion.resolve_dns, self.opts)
 
     def test_source_int_name_local(self):
         '''
@@ -43,14 +46,14 @@ class MinionTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
                                      'netmask': '111.1.0.0',
                                      'label': 'bond0',
                                      'address': '111.1.0.1'}]}}
-        with patch.dict(__opts__, {'ipv6': False, 'master': '127.0.0.1',
+        with patch.dict(self.opts, {'ipv6': False, 'master': '127.0.0.1',
                                    'master_port': '4555', 'file_client': 'local',
                                    'source_interface_name': 'bond0.1234',
                                    'source_ret_port': 49017,
                                    'source_publish_port': 49018}), \
             patch('salt.utils.network.interfaces',
                   MagicMock(return_value=interfaces)):
-            assert salt.minion.resolve_dns(__opts__) == {'master_ip': '127.0.0.1',
+            assert salt.minion.resolve_dns(self.opts) == {'master_ip': '127.0.0.1',
                                                          'source_ip': '111.1.0.1',
                                                          'source_ret_port': 49017,
                                                          'source_publish_port': 49018,
@@ -68,14 +71,14 @@ class MinionTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
                                      'netmask': '111.1.0.0',
                                      'label': 'bond0',
                                      'address': '111.1.0.1'}]}}
-        with patch.dict(__opts__, {'ipv6': False, 'master': '127.0.0.1',
+        with patch.dict(self.opts, {'ipv6': False, 'master': '127.0.0.1',
                                    'master_port': '4555', 'file_client': 'remote',
                                    'source_interface_name': 'bond0.1234',
                                    'source_ret_port': 49017,
                                    'source_publish_port': 49018}), \
             patch('salt.utils.network.interfaces',
                   MagicMock(return_value=interfaces)):
-            assert salt.minion.resolve_dns(__opts__) == {'master_ip': '127.0.0.1',
+            assert salt.minion.resolve_dns(self.opts) == {'master_ip': '127.0.0.1',
                                                          'source_ret_port': 49017,
                                                          'source_publish_port': 49018,
                                                          'master_uri': 'tcp://127.0.0.1:4555'}
@@ -90,7 +93,7 @@ class MinionTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
                                      'netmask': '111.1.0.0',
                                      'label': 'bond0',
                                      'address': '111.1.0.1'}]}}
-        with patch.dict(__opts__, {'ipv6': False, 'master': '127.0.0.1',
+        with patch.dict(self.opts, {'ipv6': False, 'master': '127.0.0.1',
                                    'master_port': '4555', 'file_client': 'local',
                                    'source_interface_name': '',
                                    'source_address': '111.1.0.1',
@@ -98,7 +101,7 @@ class MinionTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
                                    'source_publish_port': 49018}), \
             patch('salt.utils.network.interfaces',
                   MagicMock(return_value=interfaces)):
-            assert salt.minion.resolve_dns(__opts__) == {'source_publish_port': 49018,
+            assert salt.minion.resolve_dns(self.opts) == {'source_publish_port': 49018,
                                                          'source_ret_port': 49017,
                                                          'master_uri': 'tcp://127.0.0.1:4555',
                                                          'source_ip': '111.1.0.1',
@@ -116,11 +119,11 @@ class MinionTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
         return None BEFORE any of the processes are spun up because we should be avoiding firing duplicate
         jobs.
         '''
-        mock_opts = salt.config.DEFAULT_MINION_OPTS
+        mock_opts = salt.config.DEFAULT_MINION_OPTS.copy()
         mock_data = {'fun': 'foo.bar',
                      'jid': 123}
         mock_jid_queue = [123]
-        minion = salt.minion.Minion(mock_opts, jid_queue=copy.copy(mock_jid_queue), io_loop=tornado.ioloop.IOLoop())
+        minion = salt.minion.Minion(mock_opts, jid_queue=copy.copy(mock_jid_queue), io_loop=salt.ext.tornado.ioloop.IOLoop())
         try:
             ret = minion._handle_decoded_payload(mock_data).result()
             self.assertEqual(minion.jid_queue, mock_jid_queue)
@@ -134,14 +137,14 @@ class MinionTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
         jid isn't already present in the jid_queue.
         '''
         with patch('salt.minion.Minion.ctx', MagicMock(return_value={})), \
-                patch('salt.utils.process.SignalHandlingMultiprocessingProcess.start', MagicMock(return_value=True)), \
-                patch('salt.utils.process.SignalHandlingMultiprocessingProcess.join', MagicMock(return_value=True)):
+                patch('salt.utils.process.SignalHandlingProcess.start', MagicMock(return_value=True)), \
+                patch('salt.utils.process.SignalHandlingProcess.join', MagicMock(return_value=True)):
             mock_jid = 11111
-            mock_opts = salt.config.DEFAULT_MINION_OPTS
+            mock_opts = salt.config.DEFAULT_MINION_OPTS.copy()
             mock_data = {'fun': 'foo.bar',
                          'jid': mock_jid}
             mock_jid_queue = [123, 456]
-            minion = salt.minion.Minion(mock_opts, jid_queue=copy.copy(mock_jid_queue), io_loop=tornado.ioloop.IOLoop())
+            minion = salt.minion.Minion(mock_opts, jid_queue=copy.copy(mock_jid_queue), io_loop=salt.ext.tornado.ioloop.IOLoop())
             try:
 
                 # Assert that the minion's jid_queue attribute matches the mock_jid_queue as a baseline
@@ -163,14 +166,14 @@ class MinionTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
         minion's jid_queue high water mark (minion_jid_queue_hwm) is hit.
         '''
         with patch('salt.minion.Minion.ctx', MagicMock(return_value={})), \
-                patch('salt.utils.process.SignalHandlingMultiprocessingProcess.start', MagicMock(return_value=True)), \
-                patch('salt.utils.process.SignalHandlingMultiprocessingProcess.join', MagicMock(return_value=True)):
-            mock_opts = salt.config.DEFAULT_MINION_OPTS
+                patch('salt.utils.process.SignalHandlingProcess.start', MagicMock(return_value=True)), \
+                patch('salt.utils.process.SignalHandlingProcess.join', MagicMock(return_value=True)):
+            mock_opts = salt.config.DEFAULT_MINION_OPTS.copy()
             mock_opts['minion_jid_queue_hwm'] = 2
             mock_data = {'fun': 'foo.bar',
                          'jid': 789}
             mock_jid_queue = [123, 456]
-            minion = salt.minion.Minion(mock_opts, jid_queue=copy.copy(mock_jid_queue), io_loop=tornado.ioloop.IOLoop())
+            minion = salt.minion.Minion(mock_opts, jid_queue=copy.copy(mock_jid_queue), io_loop=salt.ext.tornado.ioloop.IOLoop())
             try:
 
                 # Assert that the minion's jid_queue attribute matches the mock_jid_queue as a baseline
@@ -191,31 +194,32 @@ class MinionTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
         as per process_count_max.
         '''
         with patch('salt.minion.Minion.ctx', MagicMock(return_value={})), \
-                patch('salt.utils.process.SignalHandlingMultiprocessingProcess.start', MagicMock(return_value=True)), \
-                patch('salt.utils.process.SignalHandlingMultiprocessingProcess.join', MagicMock(return_value=True)), \
+                patch('salt.utils.process.SignalHandlingProcess.start', MagicMock(return_value=True)), \
+                patch('salt.utils.process.SignalHandlingProcess.join', MagicMock(return_value=True)), \
                 patch('salt.utils.minion.running', MagicMock(return_value=[])), \
-                patch('tornado.gen.sleep', MagicMock(return_value=tornado.concurrent.Future())):
+                patch('salt.ext.tornado.gen.sleep', MagicMock(return_value=salt.ext.tornado.concurrent.Future())):
             process_count_max = 10
-            mock_opts = salt.config.DEFAULT_MINION_OPTS
+            mock_opts = salt.config.DEFAULT_MINION_OPTS.copy()
+            mock_opts['__role'] = 'minion'
             mock_opts['minion_jid_queue_hwm'] = 100
             mock_opts["process_count_max"] = process_count_max
 
-            io_loop = tornado.ioloop.IOLoop()
+            io_loop = salt.ext.tornado.ioloop.IOLoop()
             minion = salt.minion.Minion(mock_opts, jid_queue=[], io_loop=io_loop)
             try:
 
                 # mock gen.sleep to throw a special Exception when called, so that we detect it
-                class SleepCalledEception(Exception):
+                class SleepCalledException(Exception):
                     """Thrown when sleep is called"""
-                    pass
-                tornado.gen.sleep.return_value.set_exception(SleepCalledEception())
+
+                salt.ext.tornado.gen.sleep.return_value.set_exception(SleepCalledException())
 
                 # up until process_count_max: gen.sleep does not get called, processes are started normally
                 for i in range(process_count_max):
                     mock_data = {'fun': 'foo.bar',
                                  'jid': i}
                     io_loop.run_sync(lambda data=mock_data: minion._handle_decoded_payload(data))
-                    self.assertEqual(salt.utils.process.SignalHandlingMultiprocessingProcess.start.call_count, i + 1)
+                    self.assertEqual(salt.utils.process.SignalHandlingProcess.start.call_count, i + 1)
                     self.assertEqual(len(minion.jid_queue), i + 1)
                     salt.utils.minion.running.return_value += [i]
 
@@ -223,9 +227,9 @@ class MinionTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
                 mock_data = {'fun': 'foo.bar',
                              'jid': process_count_max + 1}
 
-                self.assertRaises(SleepCalledEception,
+                self.assertRaises(SleepCalledException,
                                   lambda: io_loop.run_sync(lambda: minion._handle_decoded_payload(mock_data)))
-                self.assertEqual(salt.utils.process.SignalHandlingMultiprocessingProcess.start.call_count,
+                self.assertEqual(salt.utils.process.SignalHandlingProcess.start.call_count,
                                  process_count_max)
                 self.assertEqual(len(minion.jid_queue), process_count_max + 1)
             finally:
@@ -237,11 +241,11 @@ class MinionTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
         '''
         with patch('salt.minion.Minion.ctx', MagicMock(return_value={})), \
                 patch('salt.minion.Minion.sync_connect_master', MagicMock(side_effect=RuntimeError('stop execution'))), \
-                patch('salt.utils.process.SignalHandlingMultiprocessingProcess.start', MagicMock(return_value=True)), \
-                patch('salt.utils.process.SignalHandlingMultiprocessingProcess.join', MagicMock(return_value=True)):
+                patch('salt.utils.process.SignalHandlingProcess.start', MagicMock(return_value=True)), \
+                patch('salt.utils.process.SignalHandlingProcess.join', MagicMock(return_value=True)):
             mock_opts = self.get_config('minion', from_scratch=True)
             mock_opts['beacons_before_connect'] = True
-            io_loop = tornado.ioloop.IOLoop()
+            io_loop = salt.ext.tornado.ioloop.IOLoop()
             io_loop.make_current()
             minion = salt.minion.Minion(mock_opts, io_loop=io_loop)
             try:
@@ -263,11 +267,11 @@ class MinionTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
         '''
         with patch('salt.minion.Minion.ctx', MagicMock(return_value={})), \
                 patch('salt.minion.Minion.sync_connect_master', MagicMock(side_effect=RuntimeError('stop execution'))), \
-                patch('salt.utils.process.SignalHandlingMultiprocessingProcess.start', MagicMock(return_value=True)), \
-                patch('salt.utils.process.SignalHandlingMultiprocessingProcess.join', MagicMock(return_value=True)):
+                patch('salt.utils.process.SignalHandlingProcess.start', MagicMock(return_value=True)), \
+                patch('salt.utils.process.SignalHandlingProcess.join', MagicMock(return_value=True)):
             mock_opts = self.get_config('minion', from_scratch=True)
             mock_opts['scheduler_before_connect'] = True
-            io_loop = tornado.ioloop.IOLoop()
+            io_loop = salt.ext.tornado.ioloop.IOLoop()
             io_loop.make_current()
             minion = salt.minion.Minion(mock_opts, io_loop=io_loop)
             try:
@@ -282,9 +286,136 @@ class MinionTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
             finally:
                 minion.destroy()
 
+    def test_when_ping_interval_is_set_the_callback_should_be_added_to_periodic_callbacks(self):
+        with patch('salt.minion.Minion.ctx', MagicMock(return_value={})), \
+                patch('salt.minion.Minion.sync_connect_master', MagicMock(side_effect=RuntimeError('stop execution'))), \
+                patch('salt.utils.process.SignalHandlingProcess.start', MagicMock(return_value=True)), \
+                patch('salt.utils.process.SignalHandlingProcess.join', MagicMock(return_value=True)):
+            mock_opts = self.get_config('minion', from_scratch=True)
+            mock_opts['ping_interval'] = 10
+            io_loop = salt.ext.tornado.ioloop.IOLoop()
+            io_loop.make_current()
+            minion = salt.minion.Minion(mock_opts, io_loop=io_loop)
+            try:
+                try:
+                    minion.connected = MagicMock(side_effect=(False, True))
+                    minion._fire_master_minion_start = MagicMock()
+                    minion.tune_in(start=False)
+                except RuntimeError:
+                    pass
 
-@skipIf(NO_MOCK, NO_MOCK_REASON)
-class MinionAsyncTestCase(TestCase, AdaptedConfigurationTestCaseMixin, tornado.testing.AsyncTestCase):
+                # Make sure the scheduler is initialized but the beacons are not
+                self.assertTrue('ping' in minion.periodic_callbacks)
+            finally:
+                minion.destroy()
+
+    def test_when_passed_start_event_grains(self):
+        mock_opts = self.get_config('minion', from_scratch=True)
+        mock_opts['start_event_grains'] = ["os"]
+        io_loop = salt.ext.tornado.ioloop.IOLoop()
+        io_loop.make_current()
+        minion = salt.minion.Minion(mock_opts, io_loop=io_loop)
+        try:
+            minion.tok = MagicMock()
+            minion._send_req_sync = MagicMock()
+            minion._fire_master('Minion has started', 'minion_start', include_startup_grains=True)
+            load = minion._send_req_sync.call_args[0][0]
+
+            self.assertTrue('grains' in load)
+            self.assertTrue('os' in load['grains'])
+        finally:
+            minion.destroy()
+
+    def test_when_not_passed_start_event_grains(self):
+        mock_opts = self.get_config('minion', from_scratch=True)
+        io_loop = salt.ext.tornado.ioloop.IOLoop()
+        io_loop.make_current()
+        minion = salt.minion.Minion(mock_opts, io_loop=io_loop)
+        try:
+            minion.tok = MagicMock()
+            minion._send_req_sync = MagicMock()
+            minion._fire_master('Minion has started', 'minion_start')
+            load = minion._send_req_sync.call_args[0][0]
+
+            self.assertTrue('grains' not in load)
+        finally:
+            minion.destroy()
+
+    def test_when_other_events_fired_and_start_event_grains_are_set(self):
+        mock_opts = self.get_config('minion', from_scratch=True)
+        mock_opts['start_event_grains'] = ["os"]
+        io_loop = salt.ext.tornado.ioloop.IOLoop()
+        io_loop.make_current()
+        minion = salt.minion.Minion(mock_opts, io_loop=io_loop)
+        try:
+            minion.tok = MagicMock()
+            minion._send_req_sync = MagicMock()
+            minion._fire_master('Custm_event_fired', 'custom_event')
+            load = minion._send_req_sync.call_args[0][0]
+
+            self.assertTrue('grains' not in load)
+        finally:
+            minion.destroy()
+
+    def test_minion_retry_dns_count(self):
+        '''
+        Tests that the resolve_dns will retry dns look ups for a maximum of
+        3 times before raising a SaltMasterUnresolvableError exception.
+        '''
+        with patch.dict(self.opts, {'ipv6': False, 'master': 'dummy',
+                                   'master_port': '4555',
+                                   'retry_dns': 1, 'retry_dns_count': 3}):
+            self.assertRaises(SaltMasterUnresolvableError,
+                              salt.minion.resolve_dns, self.opts)
+
+    def test_gen_modules_executors(self):
+        '''
+        Ensure gen_modules is called with the correct arguments #54429
+        '''
+        mock_opts = self.get_config('minion', from_scratch=True)
+        io_loop = salt.ext.tornado.ioloop.IOLoop()
+        io_loop.make_current()
+        minion = salt.minion.Minion(mock_opts, io_loop=io_loop)
+
+        class MockPillarCompiler(object):
+            def compile_pillar(self):
+                return {}
+
+        try:
+            with patch('salt.pillar.get_pillar', return_value=MockPillarCompiler()):
+                with patch('salt.loader.executors') as execmock:
+                    minion.gen_modules()
+            assert execmock.called_with(minion.opts, minion.functions)
+        finally:
+            minion.destroy()
+
+    @patch('salt.utils.process.default_signals')
+    def test_reinit_crypto_on_fork(self, def_mock):
+        '''
+        Ensure salt.utils.crypt.reinit_crypto() is executed when forking for new job
+        '''
+        mock_opts = self.get_config('minion', from_scratch=True)
+        mock_opts["multiprocessing"] = True
+
+        io_loop = salt.ext.tornado.ioloop.IOLoop()
+        io_loop.make_current()
+        minion = salt.minion.Minion(mock_opts, io_loop=io_loop)
+
+        job_data = {"jid": "test-jid", "fun": "test.ping"}
+
+        def mock_start(self):
+            assert len([x for x in self._after_fork_methods if x[0] == salt.utils.crypt.reinit_crypto]) == 1  # pylint: disable=comparison-with-callable
+
+        with patch.object(salt.utils.process.SignalHandlingProcess, 'start', mock_start):
+            io_loop.run_sync(lambda: minion._handle_decoded_payload(job_data))
+
+
+class MinionAsyncTestCase(TestCase, AdaptedConfigurationTestCaseMixin, salt.ext.tornado.testing.AsyncTestCase):
+
+    def setUp(self):
+        super(MinionAsyncTestCase, self).setUp()
+        self.opts = {}
+        self.addCleanup(delattr, self, 'opts')
 
     @skip_if_not_root
     def test_sock_path_len(self):
@@ -301,9 +432,9 @@ class MinionAsyncTestCase(TestCase, AdaptedConfigurationTestCaseMixin, tornado.t
             'sock_dir': os.path.join(salt.syspaths.SOCK_DIR, 'minion'),
             'extension_modules': ''
         }
-        with patch.dict(__opts__, opts):
+        with patch.dict(self.opts, opts):
             try:
-                event_publisher = event.AsyncEventPublisher(__opts__)
+                event_publisher = event.AsyncEventPublisher(self.opts)
                 result = True
             except ValueError:
                 #  There are rare cases where we operate a closed socket, especially in containers.
